@@ -138,11 +138,11 @@ namespace apg {
                         for (uint64_t xi = 0; xi < 8; xi++) {
                             if (bitmask[j] & (1ull << (xi + (yi << 3)))) {
                                 uint64_t c = 0;
-                                for (unsigned int i = 0; i < N; i++) {
+                                for (uint64_t i = 0; i < N; i++) {
                                     uint64_t rel = pptr->key.x[4*i + j];
                                     rel = rel >> xi;
                                     rel = rel >> (yi << 3);
-                                    c |= ((rel & 1) << i);
+                                    c |= ((rel & 1ull) << i);
                                 }
                                 std::pair<uint64_t, uint64_t> loc(y + yi + ((j & 2) << 2), x + xi + ((j & 1) << 3));
                                 cells.emplace(loc, c);
@@ -304,21 +304,11 @@ namespace apg {
             write_macrocell_headerless(outstream, hnode, rule);
         }
 
-        hypernode<I> read_macrocell(std::istream &instream, std::map<uint64_t, uint64_t> *lmap, std::string &rule) {
+        hypernode<I> read_macrocell(std::istream &instream, std::string &rule) {
             /*
             * Returns a hypernode representing the contents of a macrocell
             * file. This handles both 2- and n-state macrocell files using the
             * same code.
-            *
-            * lmap should be a pointer to a map which translates the states
-            * given in the macrocell file to a bit-array in memory. For
-            * instance, if state 7 should map to 'on' in layers 0, 3 and 4,
-            * then the map should have:
-            *
-            * (*lmap)[7] = 0b00011001 = 25
-            *
-            * Zero must always map to zero. If lmap is a null pointer, then it
-            * is taken to be the identity map.
             */
 
             std::string line; // line read from file
@@ -333,27 +323,38 @@ namespace apg {
 
             std::ostringstream rlestream;
 
+            RuleMapper *grm = new IdentityMapper();
+
+            int golly_numbering = 2;
+
             while (std::getline(instream, line)) {
                 // std::cout << line << std::endl;
                 if (line.empty()) {
                     continue;
                 } else if (line[0] == '[') {
-                    // header line
-                    // std::cerr << "Header line: " << line << std::endl;
-                } else if (line[0] == 'x') {
+                    if (line.find("golly") != std::string::npos) {
+                        golly_numbering = 3;
+                    }
+                } else if ((line[0] == 'x') && (reading_rle == false)) {
                     reading_rle = true;
                     auto posrule = line.find("rule");
                     if (posrule != std::string::npos) {
                         rule = sanirule(line.substr(posrule + 4));
+                        delete grm; grm = getMapper(rule);
                         std::cerr << line << " interpreted as " << rule << std::endl;
                     }
                 } else if (line[0] == '#') {
-                    // std::cerr << "Comment line: " << line << std::endl;
                     if (line.length() >= 2 && (line[1] == 'R' || line[1] == 'G')) {
                         if (line[1] == 'R') {
                             rule = sanirule(line.substr(3));
+                            delete grm; grm = getMapper(rule);
                             std::cerr << line << " interpreted as " << rule << std::endl;
                         }
+                    } else if ((line.length() > 4) && (line.substr(0, 4) == "#CLL")) {
+                        // Lifelib-specific meta-information
+                        if ((line.length() >= 20) && (line.substr(0, 20) == "#CLL state-numbering")) {
+                            golly_numbering = (line.find("golly") != std::string::npos) ? 3 : 0;
+                        }
                     } else {
                         // line is a comment
                     }
@@ -362,7 +363,7 @@ namespace apg {
                 } else {
                     nicearray<uint64_t, N> pleaf = {0ull};
                     if (line[0] == '.' || line[0] == '*' || line[0] == '$') {
-                        uint64_t lm = (lmap == 0) ? 1 : ((*lmap)[1]);
+                        uint64_t lm = 1;
                         uint64_t pl = 0;
                         uint64_t x = 0;
                         uint64_t y = 0;
@@ -397,10 +398,10 @@ namespace apg {
                         s >> log2size >> a >> b >> c >> d;
 
                         if (log2size == 1) {
-                            uint64_t ilma = (lmap == 0) ? a : ((*lmap)[a]);
-                            uint64_t ilmb = (lmap == 0) ? b : ((*lmap)[b]);
-                            uint64_t ilmc = (lmap == 0) ? c : ((*lmap)[c]);
-                            uint64_t ilmd = (lmap == 0) ? d : ((*lmap)[d]);
+                            uint64_t ilma = ((golly_numbering & 1) ? grm->golly_to_ll(a) : a);
+                            uint64_t ilmb = ((golly_numbering & 1) ? grm->golly_to_ll(b) : b);
+                            uint64_t ilmc = ((golly_numbering & 1) ? grm->golly_to_ll(c) : c);
+                            uint64_t ilmd = ((golly_numbering & 1) ? grm->golly_to_ll(d) : d);
                             for (unsigned int i = 0; i < N; i++) {
                                 pleaf.x[i] = (ilma & 1) | ((ilmb & 1) << 1) | ((ilmc & 1) << 8) | ((ilmd & 1) << 9);
                                 ilma = ilma >> 1; ilmb = ilmb >> 1; ilmc = ilmc >> 1; ilmd = ilmd >> 1;
@@ -448,11 +449,16 @@ namespace apg {
                 }
             }
 
+            hypernode<I> result_hn;
+
             if (reading_rle) {
-                return this->fromrle(rlestream.str());
+                result_hn = this->fromrle(rlestream.str(), ((golly_numbering & 2) ? grm : 0));
             } else {
-                return hypernode<I>(lastnode, log2size - 4);
+                result_hn = hypernode<I>(lastnode, log2size - 4);
             }
+
+            delete grm;
+            return result_hn;
         }
 
         // Extract the (x, y)th cell from a node:
