@@ -359,6 +359,14 @@ namespace apg {
             return basepattern<I>(lab, lab->pattern_match(hnode, hypernode<I>(0, 1), coerce(c1)), rulestring);
         }
 
+        basepattern<I> bror() {
+            return basepattern<I>(lab, lab->bror_recurse(hnode), rulestring);
+        }
+
+        basepattern<I> brand() {
+            return basepattern<I>(lab, lab->brand_recurse(hnode), rulestring);
+        }
+
         // Periodicity detection:
 
         uint64_t digest() {
@@ -515,6 +523,21 @@ namespace apg {
             return dt;
         }
 
+        bool isHexagonal() {
+
+            std::string rule = rulestring;
+            if (rule.length() == 0) { return false; }
+
+            // Remove history suffix:
+            if ((rule.length() >= 8) && (rule.substr(rule.length() - 6) == "istory")) {
+                rule = rule.substr(0, rule.length() - 7);
+            }
+
+            // Match uppercase 'H' and lowercase 'h':
+            return (((int) rule[rule.length() - 1]) & 31) == 8;
+
+        }
+
         std::string phase_wechsler() {
             /*
             * Returns the extended Wechsler format for the current phase:
@@ -533,8 +556,13 @@ namespace apg {
                     // Remove redundant '_0' suffices:
                     bwv.pop_back();
                 }
-                std::string s = wechslerise(bwv, bbox);
-                return ((s.length() > 1280) ? "#" : s);
+                if (isHexagonal()) {
+                    std::string s = wechslerhex(bwv);
+                    return ((s.length() > 1280) ? "#" : s);
+                } else {
+                    std::string s = wechslerise(bwv, bbox);
+                    return ((s.length() > 1280) ? "#" : s);
+                }
             } else {
                 return "0";
             }
