@@ -2,6 +2,16 @@ import os
 
 from collections import Sequence, defaultdict
 
+def get_symmetries(rulestring):
+
+    if rulestring[-1].lower() == 'h':
+        validsyms = ["1x256", "2x128", "4x64", "8x32", "C1", "D2_x", "D2_xo"]
+    else:
+        validsyms = ["1x256", "2x128", "4x64", "8x32", "C1", "C2_4", "C2_2", "C2_1", "C4_4", "C4_1",
+                 "D2_+2", "D2_+1", "D2_x", "D4_+4", "D4_+2", "D4_+1", "D4_x4", "D4_x1", "D8_4", "D8_1"]
+
+    return validsyms
+
 class SampleSoupList(Sequence):
 
     def __init__(self, lt, rule, symmetry, samples):
