@@ -1,6 +1,7 @@
 from .lowlevel import *
 from .pattern import Pattern
 from .samples import download_samples as dsamples
+from .samples import get_symmetries
 import os
 
 floats = {}
@@ -47,6 +48,10 @@ class Lifetree(object):
 
     def hashsoup(self, rule, symmetry, seed):
 
+        validsyms = get_symmetries(rule)
+        if symmetry not in validsyms:
+            raise ValueError("%s is not one of the supported symmetries: %s" % (symmetry, validsyms))
+
         pptr = self.lifelib('Hashsoup', self.ptr, rule, symmetry, seed)
         return Pattern(self.session, pptr, self)
 
